@@ -20,31 +20,52 @@
 from __future__ import division
 
 import datetime
+import re
 
 
 TEXT_RESULT_LENGTH = 42
 
+ALPHA = r'[^\W\d_]'
+ALPHA_NUMERIC = r'\w'
+HEX = r'[0-9A-F]{6}'
+HASHTAG_EXCLUDES = r'%(HEX)s|include' % locals()
+HASHTAG_TEXT = r'%(ALPHA_NUMERIC)s*%(ALPHA)s+%(ALPHA_NUMERIC)s*' % locals()
+HASHTAG_PATTERN = r'(^|[^%(ALPHA_NUMERIC)s&#]+)(#|\uFF03)(?!%(HASHTAG_EXCLUDES)s)(%(HASHTAG_TEXT)s)' % locals()
+HASHTAG = re.compile(HASHTAG_PATTERN, flags=re.I | re.U)
+
+def escape_tag(tag):
+    return tag.lower().replace(' ', '_')
+
 
 def get_text_with_dots(text, start, end, found_text=None):
     '''
-    Find the outermost spaces and add dots if needed
+    Find the outermost spaces and innermost newlines around (start, end) and add
+    dots if needed.
     '''
-    bound1 = max(0, start - int(TEXT_RESULT_LENGTH//2))
-    bound2 = start
-    bound3 = end
-    bound4 = min(len(text), end + int(TEXT_RESULT_LENGTH//2))
+    bound1 = max(0, start - int(TEXT_RESULT_LENGTH // 2))
+    bound2 = max(0, start)
+    bound3 = min(end, len(text))
+    bound4 = min(len(text), end + int(TEXT_RESULT_LENGTH // 2))
 
-    if bound1 == 0:
-        start = 0
-    else:
-        start = max(bound1, text.find(' ', bound1, bound2))
+    start_values = [bound1]
+    newline = text.rfind('\n', bound1, bound2)
+    start_values.append(newline)
+    if newline == -1:
+        start_values.append(text.find(' ', bound1, bound2))
+    start = max(start_values)
 
-    if bound4 == len(text):
-        end = len(text)
+    end_values = [bound4]
+    newline = text.find('\n', bound3, bound4)
+    if newline != -1:
+        end_values.append(newline)
     else:
-        end = text.rfind(' ', bound3, bound4)
-        if end == -1:
-            end = bound4
+        space = text.rfind(' ', bound3, bound4)
+        if space != -1:
+            end_values.append(space)
+    end = min(end_values)
+
+    assert bound1 <= start <= bound2
+    assert bound3 <= end <= bound4, (bound3, end, bound4)
 
     res = ''
     if start > 0:
@@ -62,14 +83,21 @@ def get_text_with_dots(text, start, end, found_text=None):
 
 
 class Day(object):
-    def __init__(self, month, day_number, day_content = None):
-        if day_content == None:
+    def __init__(self, month, day_number, day_content=None):
+        if day_content is None:
             day_content = {'text': u''}
 
         self.date = datetime.date(month.year_number, month.month_number, day_number)
 
         self.month = month
         self.day_number = day_number
+
+        # Turn all entries of old "Tags" categories into tags without entries.
+        old_tags = day_content.pop('Tags', {})
+        for old_tag in old_tags.keys():
+            day_content[old_tag] = None
+            self.month.edited = True
+
         self.content = day_content
 
         # Remember the last edit and preview position
@@ -78,40 +106,26 @@ class Day(object):
 
     # Text
     def _get_text(self):
-        '''
-        Returns the day's text encoded as UTF-8
-        decode means "decode from the standard ascii representation"
-        '''
+        '''Return the day's text as unicode.'''
         if 'text' in self.content:
             return self.content['text'].decode('utf-8')
         else:
-           return ''
+            return ''
 
     def _set_text(self, text):
         self.content['text'] = text
     text = property(_get_text, _set_text)
 
-    def _has_text(self):
+    @property
+    def has_text(self):
         return len(self.text.strip()) > 0
-    has_text = property(_has_text)
 
 
     @property
     def empty(self):
-        if len(self.content.keys()) == 0:
+        if len(self.content) == 0:
             return True
-        elif len(self.content.keys()) == 1 and 'text' in self.content and not self.has_text:
-            return True
-        else:
-            return False
-
-
-    @property
-    def tree(self):
-        tree = self.content.copy()
-        if 'text' in tree:
-            del tree['text']
-        return tree
+        return self.content.keys() == ['text'] and not self.has_text
 
 
     def add_category_entry(self, category, entry):
@@ -141,29 +155,36 @@ class Day(object):
             for entry in entries:
                 self.add_category_entry(category, entry)
 
+    @property
+    def hashtags(self):
+        # The same tag can occur multiple times.
+        return [hashtag.lower() for _, _hash, hashtag in HASHTAG.findall(self.text)]
 
     @property
     def categories(self):
-        return self.tree.keys()
+        return self.get_category_content_pairs().keys()
 
 
     def get_entries(self, category):
-        return sorted(self.get_category_content_pairs().get(category, []))
+        entries = self.content.get(category) or {}
+        return sorted(entries.keys())
 
 
     def get_category_content_pairs(self):
         '''
         Returns a dict of (category: content_in_category_as_list) pairs.
         '''
-        original_tree = self.tree.copy()
         pairs = {}
-        #TODO: Rewrite
-        for category, content in original_tree.iteritems():
-            entry_list = []
-            if content is not None:
-                for entry, nonetype in content.iteritems():
-                    entry_list.append(entry)
-            pairs[category] = entry_list
+        for category, content in self.content.iteritems():
+            if category == 'text':
+                continue
+            if content is None:
+                pairs[category] = []
+            else:
+                pairs[category] = content.keys()
+        # Include hashtags
+        for tag in self.hashtags:
+            pairs[tag] = []
         return pairs
 
 
@@ -185,37 +206,40 @@ class Day(object):
 
 
     def search(self, text, tags):
+        """
+        Only days that have all tags are searched.
+        Search in date first, then in the text, then in the tags.
+        Uses case-insensitive search.
+        """
+        results = []
         if not text:
-            results = []
-            for day_tag in self.categories:
+            # Only add text result once for all tags.
+            add_text_to_results = False
+            for day_tag, entries in self.get_category_content_pairs().items():
                 for tag in tags:
-                    if day_tag.replace(' ', '').lower() != tag:
+                    # We know that all tags are present, but we loop through
+                    # day_tags nonetheless, to escape the day_tags.
+                    if escape_tag(day_tag) != tag:
                         continue
-                    entries = self.get_entries(day_tag)
                     if entries:
                         results.extend(entries)
                     else:
-                        results.append(get_text_with_dots(self.text, 0,
-                                       TEXT_RESULT_LENGTH))
-            return str(self), results
+                        add_text_to_results = True
+            if add_text_to_results:
+                results.append(get_text_with_dots(self.text, 0, TEXT_RESULT_LENGTH))
 
-        results = []
-        # Search in date
-        if text in str(self):
+        elif text in str(self):
+            # Search in date
             results.append(get_text_with_dots(self.text, 0, TEXT_RESULT_LENGTH))
-            return results
-        text_result = self.search_in_text(text)
-        if text_result:
-            results.append(text_result)
-        results.extend(self.search_in_categories(text))
+        else:
+            text_result = self.search_in_text(text)
+            if text_result:
+                results.append(text_result)
+            results.extend(self.search_in_categories(text))
         return str(self), results
 
 
     def search_in_text(self, search_text):
-        '''
-        Try searching in date first, then in the text, then in the annotations
-        Uses case-insensitive search
-        '''
         occurence = self.text.upper().find(search_text.upper())
 
         # Check if search_text is in text
@@ -224,7 +248,8 @@ class Day(object):
 
         found_text = self.text[occurence:occurence + len(search_text)]
         result_text = get_text_with_dots(self.text, occurence,
-                                    occurence + len(search_text), found_text)
+                                         occurence + len(search_text),
+                                         found_text)
         return result_text
 
 
@@ -252,18 +277,18 @@ class Day(object):
 
 
 class Month(object):
-    def __init__(self, year_number, month_number, month_content = None):
-        if month_content == None:
+    def __init__(self, year_number, month_number, month_content=None):
+        if month_content is None:
             month_content = {}
 
+        self.edited = False
+
         self.year_number = year_number
         self.month_number = month_number
         self.days = {}
         for day_number, day_content in month_content.iteritems():
             self.days[day_number] = Day(self, day_number, day_content)
 
-        self.edited = False
-
     def get_day(self, day_number):
         if day_number in self.days:
             return self.days[day_number]
@@ -286,11 +311,11 @@ class Month(object):
         return True
 
     def same_month(date1, date2):
-        if date1 == None or date2 == None:
+        if date1 is None or date2 is None:
             return False
         return date1.month == date2.month and date1.year == date2.year
     same_month = staticmethod(same_month)
 
     def __cmp__(self, other):
         return cmp((self.year_number, self.month_number),
-                    (other.year_number, other.month_number))
+                   (other.year_number, other.month_number))
