@@ -36,6 +36,7 @@ class CategoriesTreeView(object):
 
         # Maintain a list of all entered categories. Initialized by rn.__init__()
         self.categories = []
+        self.last_category = ''
 
         self.statusbar = self.main_window.statusbar
 
@@ -78,7 +79,7 @@ class CategoriesTreeView(object):
 
         # Enable a context menu
         self.context_menu = self._get_context_menu()
-        self.context_menu.attach_to_widget(self.tree_view, lambda x,y:None)
+        self.context_menu.attach_to_widget(self.tree_view, lambda x, y: None)
 
         self.tree_view.connect('button-press-event', self.on_button_press_event)
         self.tree_view.connect('key-press-event', self.on_key_press_event)
@@ -88,8 +89,13 @@ class CategoriesTreeView(object):
         self.cell.props.wrap_width = 200
         self.tree_view.connect_after("size-allocate", self.on_size_allocate, self.tvcolumn, self.cell)
 
+    def _show_error_msg(self, text):
+        self.main_window.journal.show_message(text, error=True)
+
     def add_category(self, category):
         """Add a new category name and sort all categories."""
+        if category:
+            self.last_category = category
         if category is None or category in self.categories:
             return
         self.categories.append(category)
@@ -124,10 +130,10 @@ class CategoriesTreeView(object):
         new_text is txt2tags markup
         """
         if new_text == 'text' and self.node_on_top_level(path):
-            self.statusbar.show_text('"text" is a reserved keyword', error=True)
+            self._show_error_msg('"text" is a reserved keyword')
             return
         if len(new_text) < 1:
-            self.statusbar.show_text(_('Empty entries are not allowed'), error=True)
+            self._show_error_msg(_('Empty entries are not allowed'))
             return
 
         liststore[path][0] = markup.convert_to_pango(new_text)
@@ -141,11 +147,9 @@ class CategoriesTreeView(object):
 
     def check_category(self, category):
         if category == 'text':
-            self.statusbar.show_text('"text" is a reserved keyword', error=True)
-            return False
-        if len(category) < 1:
-            self.statusbar.show_text(_('Empty tag names are not allowed'), error=True)
+            self._show_error_msg('"text" is a reserved keyword')
             return False
+        assert category
         return True
 
     def add_element(self, parent, element_content):
@@ -159,7 +163,7 @@ class CategoriesTreeView(object):
             if key is not None:
                 key_pango = markup.convert_to_pango(key)
             new_child = self.tree_store.append(parent, [key_pango])
-            if not value == None:
+            if value is not None:
                 self.add_element(new_child, value)
 
     def set_day_content(self, day):
@@ -225,11 +229,15 @@ class CategoriesTreeView(object):
         pango_markup = markup.convert_to_pango(txt2tags_markup)
         self.tree_store.set_value(iter, 0, pango_markup)
 
-    def find_iter(self, category, entry):
+    def find_iter(self, category, entry=None):
         logging.debug('Looking for iter: "%s", "%s"' % (category, entry))
         category_iter = self._get_category_iter(category)
 
-        if not category_iter:
+        if category_iter:
+            # If we only search the category, return it.
+            if not entry:
+                return category_iter
+        else:
             # If the category was not found, return None
             return None
 
@@ -362,7 +370,7 @@ class CategoriesTreeView(object):
         """
         #Get the path at the specific mouse position
         path = widget.get_path_at_pos(int(event.x), int(event.y))
-        if (path == None):
+        if (path is None):
             """If we didn't get a path then we don't want anything
             to be selected."""
             selection = widget.get_selection()
@@ -397,19 +405,13 @@ class CategoriesTreeView(object):
 
         # Create actions
         actiongroup.add_actions([
-            ('ChangeEntry', gtk.STOCK_EDIT,
-                _('Change this text'),
-                None, None, self._on_change_entry_clicked
-            ),
-            ('AddEntry', gtk.STOCK_NEW,
-                _('Add a new entry'),
-                None, None, self._on_add_entry_clicked
-            ),
-            ('Delete', gtk.STOCK_DELETE,
-                _('Delete this entry'),
-                None, None, self._on_delete_entry_clicked
-            ),
-            ])
+            ('ChangeEntry', gtk.STOCK_EDIT, _('Change this text'),
+             None, None, self._on_change_entry_clicked),
+            ('AddEntry', gtk.STOCK_NEW, _('Add a new entry'),
+             None, None, self._on_add_entry_clicked),
+            ('Delete', gtk.STOCK_DELETE, _('Delete this entry'),
+             None, None, self._on_delete_entry_clicked),
+        ])
 
         # Add the actiongroup to the uimanager
         uimanager.insert_action_group(actiongroup, 0)
@@ -471,7 +473,7 @@ class CategoriesTreeView(object):
         while iter and store.iter_is_valid(iter):
             store.row_changed(store.get_path(iter), iter)
             iter = store.iter_next(iter)
-        treeview.set_size_request(0,-1)
+        treeview.set_size_request(0, -1)
 
         ## The heights may have changed
         column.queue_resize()
