@@ -42,8 +42,6 @@ class CustomComboBoxEntry(object):
     def __init__(self, combo_box):
         self.combo_box = combo_box
 
-        #self.liststore = self.combo_box.get_model()
-        #if self.liststore is None:
         self.liststore = gtk.ListStore(gobject.TYPE_STRING)
         self.combo_box.set_model(self.liststore)
         #self.combo_box.set_wrap_width(5)
@@ -62,8 +60,10 @@ class CustomComboBoxEntry(object):
 
     def set_entries(self, value_list):
         self.clear()
+        self.combo_box.set_model(None)
         for entry in value_list:
             self.add_entry(entry)
+        self.combo_box.set_model(self.liststore)
 
     def get_active_text(self):
         return self.entry.get_text().decode('utf-8')
@@ -72,9 +72,11 @@ class CustomComboBoxEntry(object):
         return self.entry.set_text(text)
 
     def clear(self):
+        self.combo_box.set_model(None)
         if self.liststore:
             self.liststore.clear()
         self.set_active_text('')
+        self.combo_box.set_model(self.liststore)
 
     def connect(self, *args, **kargs):
         self.combo_box.connect(*args, **kargs)
@@ -84,12 +86,16 @@ class CustomComboBoxEntry(object):
 
 
 class CustomListView(gtk.TreeView):
-    def __init__(self):
+    def __init__(self, columns):
+        """
+        *columns* must be a list of (header, type) pairs e.g. [('title', str)].
+        """
         gtk.TreeView.__init__(self)
-        # create a TreeStore with two string columns to use as the model
-        self.set_model(gtk.ListStore(str, str))
+        headers, types = zip(*columns)
+        # create a TreeStore with columns to use as the model
+        self.set_model(gtk.ListStore(*types))
 
-        columns = [gtk.TreeViewColumn('1'), gtk.TreeViewColumn('2')]
+        columns = [gtk.TreeViewColumn(header) for header in headers]
 
         # add tvcolumns to tree_view
         for index, column in enumerate(columns):
@@ -126,14 +132,58 @@ class Calendar(gtk.Calendar):
         self.select_day(1)
 
         # PyGTK calendars show months in range [0,11]
-        self.select_month(date.month-1, date.year)
+        self.select_month(date.month - 1, date.year)
 
         # Select the day after the month and year have been set
         self.select_day(date.day)
 
     def get_date(self):
         year, month, day = gtk.Calendar.get_date(self)
-        return datetime.date(year, month+1, day)
+        return datetime.date(year, month + 1, day)
+
+
+# gtk.InfoBar is available in gtk+ >= 2.22
+if hasattr(gtk, 'InfoBar'):
+    class Info(gtk.InfoBar):
+        icons = {gtk.MESSAGE_ERROR: gtk.STOCK_DIALOG_ERROR}
+
+        def __init__(self, *args, **kwargs):
+            gtk.InfoBar.__init__(self, *args, **kwargs)
+            self.title_label = gtk.Label()
+            self.msg_label = gtk.Label()
+            self.title_label.set_alignment(0., 0.5)
+            self.msg_label.set_alignment(0., 0.5)
+
+            vbox = gtk.VBox(spacing=5)
+            vbox.pack_start(self.title_label, False, False)
+            vbox.pack_start(self.msg_label, False, False)
+
+            self.image = gtk.Image()
+
+            content = self.get_content_area()
+            content.pack_start(self.image, False)
+            content.pack_start(vbox, False)
+
+            self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+            self.connect('close', lambda x: self.hide())
+            self.connect('response', self.on_response)
+
+        def on_response(self, infobar, response_id):
+            if response_id == gtk.RESPONSE_CLOSE:
+                self.hide()
+
+        def show_message(self, title, msg, msg_type):
+            if not title:
+                title = msg
+                msg = ''
+            self.title_label.set_markup('<b>%s</b>' % title)
+            self.msg_label.set_markup(msg)
+            self.set_message_type(msg_type)
+            self.image.set_from_stock(self.icons.get(msg_type, gtk.STOCK_DIALOG_INFO),
+                                      gtk.ICON_SIZE_DIALOG)
+            self.show_all()
+else:
+    Info = None
 
 
 # ------------------------- Assistant Pages ------------------------------------
@@ -198,7 +248,7 @@ class RadioButtonPage(AssistantPage):
         if tooltip:
             description = gtk.Label()
             description.set_alignment(0.0, 0.5)
-            description.set_markup(' '*5 + tooltip)
+            description.set_markup(' ' * 5 + tooltip)
             description.set_sensitive(sensitive)
             self.pack_start(description, False, False)
 
@@ -316,3 +366,38 @@ class Assistant(gtk.Assistant):
         self.set_page_title(page, _('Introduction'))
         self.set_page_type(page, gtk.ASSISTANT_PAGE_INTRO)
         self.set_page_complete(page, True)
+
+
+class TemplateBar(gtk.HBox):
+    def __init__(self):
+        gtk.HBox.__init__(self)
+        self.set_spacing(2)
+        label = gtk.Label('<b>%s</b>:' % _('Template'))
+        label.set_use_markup(True)
+        self.pack_start(label, False, False)
+        self.save_insert_button = gtk.Button(_('Save and insert'))
+        self.pack_start(self.save_insert_button, False, False)
+        self.save_button = gtk.Button(stock=gtk.STOCK_SAVE)
+        self.pack_start(self.save_button, False, False)
+        self.close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
+        self.pack_start(self.close_button, False, False)
+        self.show_all()
+
+
+class ToolbarMenuButton(gtk.MenuToolButton):
+    def __init__(self, stock_id, menu):
+        gtk.MenuToolButton.__init__(self, stock_id)
+        self.set_menu(menu)
+        self.connect('clicked', self.show_menu)
+        self.show()
+
+    def show_menu(self, button):
+        """
+        Show the menu when the button is clicked.
+
+        A little hack for button and activate_time is needed as the "clicked" does
+        not have an associated event parameter. Otherwise we would use event.button
+        and event.time
+        """
+        self.get_menu().popup(parent_menu_shell=None, parent_menu_item=None,
+                              func=None, button=0, activate_time=0, data=None)
