@@ -25,7 +25,6 @@ import gtk
 import gobject
 import pango
 
-# try to import gtkspell
 try:
     import gtkspell
 except ImportError:
@@ -33,6 +32,10 @@ except ImportError:
 
 from rednotebook.gui import t2t_highlight
 from rednotebook import undo
+from rednotebook.util import filesystem
+
+
+DEFAULT_FONT = gtk.settings_get_default().get_property('gtk-font-name')
 
 
 class Editor(object):
@@ -64,8 +67,7 @@ class Editor(object):
         # So we forbid that behaviour, by setting a minimum width
         self.day_text_view.set_size_request(1, -1)
 
-        font_name = gtk.settings_get_default().get_property('gtk-font-name')
-        self.font = pango.FontDescription(font_name)
+        self.font = pango.FontDescription(DEFAULT_FONT)
         self.default_size = self.font.get_size() / pango.SCALE
         logging.debug('Default font: %s' % self.font.to_string())
         logging.debug('Default size: %s' % self.default_size)
@@ -97,6 +99,27 @@ class Editor(object):
         self.day_text_buffer.handler_unblock(self.changed_connection)
         self.on_text_change(self.day_text_buffer, undoing=undoing)
 
+    def replace_selection(self, text):
+        self.force_adding_undo_point = True
+
+        self.day_text_buffer.delete_selection(interactive=False,
+                                              default_editable=True)
+        self.day_text_buffer.insert_at_cursor(text)
+
+    def replace_selection_and_highlight(self, p1, p2, p3):
+        """
+        Insert all three parts and highlight the middle part.
+        """
+        self.replace_selection(p1 + p2 + p3)
+        # Get the mark at the end of the insertion.
+        insert_mark = self.day_text_buffer.get_insert()
+        insert_iter = self.day_text_buffer.get_iter_at_mark(insert_mark)
+        start = insert_iter.copy()
+        end = insert_iter.copy()
+        start.backward_chars(len(p3) + len(p2))
+        end.backward_chars(len(p3))
+        self.day_text_buffer.select_range(start, end)
+
     def highlight(self, text):
         self.search_text = text
         self.day_text_buffer.set_search_text(text)
@@ -126,7 +149,7 @@ class Editor(object):
         if bounds:
             return self.get_text(*bounds)
         else:
-            return None
+            return ''
 
     def get_text_around_selected_text(self, length):
         bounds = self.get_selection_bounds()
@@ -138,14 +161,9 @@ class Editor(object):
         end2.forward_chars(length)
         return (self.get_text(start1, start2), self.get_text(end1, end2))
 
-    def set_selection(self, iter1, iter2):
-        '''
-        Sort the two iters and select the text between them
-        '''
-        sort_by_position = lambda iter: iter.get_offset()
-        iter1, iter2 = sorted([iter1, iter2], key=sort_by_position)
-        assert iter1.get_offset() <= iter2.get_offset()
-        self.day_text_buffer.select_range(iter1, iter2)
+    @staticmethod
+    def sort_iters(*iters):
+        return sorted(iters, key=lambda iter: iter.get_offset())
 
     def get_selection_bounds(self):
         '''
@@ -156,62 +174,34 @@ class Editor(object):
         '''
         mark1 = self.day_text_buffer.get_insert()
         mark2 = self.day_text_buffer.get_selection_bound()
-
         iter1 = self.day_text_buffer.get_iter_at_mark(mark1)
         iter2 = self.day_text_buffer.get_iter_at_mark(mark2)
+        return self.sort_iters(iter1, iter2)
 
-        sort_by_position = lambda iter: iter.get_offset()
-        iter1, iter2 = sorted([iter1, iter2], key=sort_by_position)
+    def get_text_parts(self):
+        """
+        Return text before the selection, the selected text itself and
+        the text after the selection.
+        """
+        start = self.day_text_buffer.get_start_iter()
+        end = self.day_text_buffer.get_end_iter()
+        sel_start, sel_end = self.get_selection_bounds()
+        return (self.get_text(start, sel_start),
+                self.get_text(sel_start, sel_end),
+                self.get_text(sel_end, end))
 
-        assert iter1.get_offset() <= iter2.get_offset()
-        return (iter1, iter2)
-
-    def apply_format(self, format, markup):
+    def apply_format(self, markup):
         text_around_selection = self.get_text_around_selected_text(2)
         # Apply formatting only once if a format button is clicked multiple times
         if text_around_selection == (unicode(markup), unicode(markup)):
             return
 
-        selected_text = self.get_selected_text()
+        text = self.get_selected_text() or ' '
+        self.replace_selection_and_highlight(markup, text, markup)
 
-        # If no text has been selected add example text and select it
-        if not selected_text:
-            selected_text = ' '  #'%s text' % format
-            self.insert(selected_text)
-
-            # Set the selection to the new text
-
-            # get_insert() returns the position of the cursor (after 2nd markup)
-            insert_mark = self.day_text_buffer.get_insert()
-            insert_iter = self.day_text_buffer.get_iter_at_mark(insert_mark)
-            markup_start_iter = insert_iter.copy()
-            markup_end_iter = insert_iter.copy()
-            markup_start_iter.backward_chars(len(selected_text))
-            markup_end_iter.backward_chars(0)
-            self.set_selection(markup_start_iter, markup_end_iter)
-
-        # Check that there is a selection
-        assert self.day_text_buffer.get_selection_bounds()
-
-        # Add the markup around the selected text
-        insert_bound = self.day_text_buffer.get_insert()
-        selection_bound = self.day_text_buffer.get_selection_bound()
-        self.insert(markup, insert_bound)
-        self.insert(markup, selection_bound)
-
-        # Set the selection to the formatted text
-        iter1, iter2 = self.get_selection_bounds()
-        selection_start_iter = iter2.copy()
-        selection_end_iter = iter2.copy()
-        selection_start_iter.backward_chars(len(selected_text) + len(markup))
-        selection_end_iter.backward_chars(len(markup))
-        self.set_selection(selection_start_iter, selection_end_iter)
-
-    def set_font_size(self, size):
-        if size <= 0:
-            size = self.default_size
-        self.font.set_size(size * pango.SCALE)
-        self.day_text_view.modify_font(self.font)
+    def set_font(self, font_name):
+        font = pango.FontDescription(font_name)
+        self.day_text_view.modify_font(font)
 
     def hide(self):
         self.day_text_view.hide()
@@ -258,9 +248,17 @@ class Editor(object):
             if self._spell_checker is None:
                 try:
                     self._spell_checker = gtkspell.Spell(self.day_text_view)
-                except gobject.GError, err:
+                except gobject.GError as err:
                     logging.error('Spell checking could not be enabled: "%s"' % err)
                     self._spell_checker = None
+                if self._spell_checker:
+                    try:
+                        self._spell_checker.set_language(filesystem.LANGUAGE)
+                    except RuntimeError as err:
+                        logging.error('Spellchecking could not be enabled for %s: %s' %
+                                      (filesystem.LANGUAGE, err))
+                        logging.error('Consult built-in help for instructions '
+                                      'on how to add custom dictionaries.')
         else:
             if self._spell_checker is not None:
                 self._spell_checker.detach()
@@ -268,7 +266,7 @@ class Editor(object):
 
     def is_spell_check_enabled(self):
         """Returns True if spell check is enabled"""
-        return self._spell_checker != None
+        return self._spell_checker is not None
 
     #===========================================================
 
@@ -289,7 +287,7 @@ class Editor(object):
 
         uris = selection.data.strip('\r\n\x00')
         logging.debug('URIs: "%s"' % uris)
-        uris = uris.split() # we may have more than one file dropped
+        uris = uris.split()  # we may have more than one file dropped
         uris = map(lambda uri: uri.strip(), uris)
         for uri in uris:
             uri = urllib.url2pathname(uri)
