@@ -34,13 +34,13 @@ if __name__ == '__main__':
 from rednotebook.util import filesystem
 from rednotebook.util import markup
 from rednotebook.util import dates
+from rednotebook.gui import customwidgets
 from rednotebook.gui.customwidgets import Calendar, AssistantPage, \
-                                    RadioButtonPage, PathChooserPage, Assistant
+    RadioButtonPage, PathChooserPage, Assistant
 from rednotebook.gui import browser
 from rednotebook.gui import options
 
 
-
 class DatePage(AssistantPage):
     def __init__(self, journal, *args, **kwargs):
         AssistantPage.__init__(self, *args, **kwargs)
@@ -48,15 +48,18 @@ class DatePage(AssistantPage):
         self.journal = journal
 
         self.all_days_button = gtk.RadioButton(label=_('Export all days'))
+        self.selected_text_button = gtk.RadioButton(
+            group=self.all_days_button)
         self.one_day_button = gtk.RadioButton(
-                                    label=_('Export currently visible day'),
-                                    group=self.all_days_button)
+            label=_('Export currently visible day'),
+            group=self.all_days_button)
         self.sel_days_button = gtk.RadioButton(
-                    label=_('Export days in the selected time range'),
-                    group=self.all_days_button)
+            label=_('Export days in the selected time range'),
+            group=self.all_days_button)
 
         self.pack_start(self.all_days_button, False)
         self.pack_start(self.one_day_button, False)
+        self.pack_start(self.selected_text_button, False)
         self.pack_start(self.sel_days_button, False)
 
         label1 = gtk.Label()
@@ -84,32 +87,43 @@ class DatePage(AssistantPage):
         self.all_days_button.set_active(True)
         self._set_select_days(False)
 
-
     def _on_select_days_toggled(self, button):
         select = self.sel_days_button.get_active()
         self._set_select_days(select)
 
-
     def _set_select_days(self, sensitive):
         self.calendar1.set_sensitive(sensitive)
         self.calendar2.set_sensitive(sensitive)
         self.select_days = sensitive
 
-
     def export_all_days(self):
         return self.all_days_button.get_active()
 
+    def export_selected_text(self):
+        return self.selected_text_button.get_active()
 
     def get_date_range(self):
         if self.select_days:
             return (self.calendar1.get_date(), self.calendar2.get_date())
         return (self.journal.day.date,) * 2
 
-
     def refresh_dates(self):
         self.calendar1.set_date(datetime.date.today())
         self.calendar2.set_date(datetime.date.today())
 
+    def prepare(self):
+        selected_text_label = _('Export currently selected text')
+        self.selected_text = self.journal.frame.day_text_field.get_selected_text()
+        enable_selected_text_button = bool(self.selected_text and not self.journal.frame.preview_mode)
+        self.selected_text_button.set_sensitive(enable_selected_text_button)
+        if enable_selected_text_button:
+            self.selected_text_button.set_label(selected_text_label)
+        else:
+            self.selected_text_button.set_label(selected_text_label + ' ' +
+                _('(Only available when text is selected in edit mode)'))
+            if self.selected_text_button.get_active():
+                self.selected_text_button.set_active(False)
+                self.all_days_button.set_active(True)
 
 
 class ContentsPage(AssistantPage):
@@ -126,34 +140,28 @@ class ContentsPage(AssistantPage):
         self.date_format = options.DateFormatOption(_('Date format'), 'exportDateFormat')
         self.date_format.combo.combo_box.set_tooltip_text(_('Leave blank to omit dates in export'))
 
-        self.text_button = gtk.CheckButton(label=_('Export texts'))
-        self.all_categories_button = gtk.RadioButton(label=_('Export all tags'))
-        self.no_categories_button = gtk.RadioButton(label=_('Do not export tags'),
-                                            group=self.all_categories_button)
-        self.sel_categories_button = gtk.RadioButton(label=_('Export only the selected tags'),
-                                            group=self.all_categories_button)
+        self.text_and_tags_button = gtk.RadioButton(label=_('Export text and tags'))
+        self.text_only_button = gtk.RadioButton(label=_('Export text only'),
+                                                group=self.text_and_tags_button)
+        self.tags_only_button = gtk.RadioButton(label=_('Export tags only'),
+                                                group=self.text_and_tags_button)
+        self.filter_tags_button = gtk.CheckButton(label=_('Filter days by tags'))
 
         self.pack_start(self.date_format, False)
-        self.pack_start(self.text_button, False)
-        self.pack_start(self.all_categories_button, False)
-        self.pack_start(self.no_categories_button, False)
-        self.pack_start(self.sel_categories_button, False)
+        self.pack_start(self.text_and_tags_button, False)
+        self.pack_start(self.text_only_button, False)
+        self.pack_start(self.tags_only_button, False)
+        self.pack_start(gtk.HSeparator(), False)
+        self.pack_start(self.filter_tags_button, False)
 
-        self.available_categories = gtk.TreeView()
+        self.available_categories = customwidgets.CustomListView([(_('Available tags'), str)])
+        self.selected_categories = customwidgets.CustomListView([(_('Selected tags'), str)])
 
-        column = gtk.TreeViewColumn(_('Available tags'))
-        self.available_categories.append_column(column)
-        cell = gtk.CellRendererText()
-        column.pack_start(cell, True)
-        column.add_attribute(cell, 'text', 0)
+        left_scroll = gtk.ScrolledWindow()
+        left_scroll.add(self.available_categories)
 
-        self.selected_categories = gtk.TreeView()
-
-        column = gtk.TreeViewColumn(_('Selected tags'))
-        self.selected_categories.append_column(column)
-        cell = gtk.CellRendererText()
-        column.pack_start(cell, True)
-        column.add_attribute(cell, 'text', 0)
+        right_scroll = gtk.ScrolledWindow()
+        right_scroll.add(self.selected_categories)
 
         self.select_button = gtk.Button(_('Select') + ' >>')
         self.deselect_button = gtk.Button('<< ' + _('Deselect'))
@@ -169,9 +177,9 @@ class ContentsPage(AssistantPage):
         vbox.pack_start(centered_vbox, True, False)
 
         hbox = gtk.HBox()
-        hbox.pack_start(self.available_categories)
+        hbox.pack_start(left_scroll)
         hbox.pack_start(vbox, False)
-        hbox.pack_start(self.selected_categories)
+        hbox.pack_start(right_scroll)
         self.pack_start(hbox)
 
         self.error_text = gtk.Label('')
@@ -179,30 +187,23 @@ class ContentsPage(AssistantPage):
 
         self.pack_end(self.error_text, False, False)
 
-        self.text_button.set_active(True)
-
-        self.text_button.connect('toggled', self.check_selection)
-        self.all_categories_button.connect('toggled', self.check_selection)
-        self.no_categories_button.connect('toggled', self.check_selection)
-        self.sel_categories_button.connect('toggled', self.check_selection)
-
+        self.text_and_tags_button.set_active(True)
+        self.filter_tags_button.connect('toggled', self.check_selection)
 
     def refresh_categories_list(self):
         model_available = gtk.ListStore(gobject.TYPE_STRING)
         for category in self.journal.categories:
-            new_row = model_available.insert(0)
-            model_available.set(new_row, 0, category)
+            model_available.append([category])
         self.available_categories.set_model(model_available)
 
         model_selected = gtk.ListStore(gobject.TYPE_STRING)
         self.selected_categories.set_model(model_selected)
 
-
     def on_select_category(self, widget):
         selection = self.available_categories.get_selection()
         nb_selected, selected_iter = selection.get_selected()
 
-        if selected_iter != None :
+        if selected_iter is not None:
             model_available = self.available_categories.get_model()
             model_selected = self.selected_categories.get_model()
 
@@ -215,12 +216,11 @@ class ContentsPage(AssistantPage):
 
         self.check_selection()
 
-
     def on_deselect_category(self, widget):
         selection = self.selected_categories.get_selection()
         nb_selected, selected_iter = selection.get_selected()
 
-        if selected_iter != None :
+        if selected_iter is not None:
             model_available = self.available_categories.get_model()
             model_selected = self.selected_categories.get_model()
 
@@ -233,20 +233,21 @@ class ContentsPage(AssistantPage):
 
         self.check_selection()
 
-
     def set_error_text(self, text):
         self.error_text.set_markup('<b>' + text + '</b>')
 
+    def is_text_included(self):
+        return self.text_only_button.get_active() or self.text_and_tags_button.get_active()
 
-    def is_text_exported(self):
-        return self.text_button.get_active()
+    def is_tags_included(self):
+        return self.tags_only_button.get_active() or self.text_and_tags_button.get_active()
 
+    def is_filtered(self):
+        return self.filter_tags_button.get_active()
 
     def get_categories(self):
-        if self.all_categories_button.get_active():
+        if not self.filter_tags_button.get_active():
             return self.journal.categories
-        elif self.no_categories_button.get_active():
-            return []
         else:
             selected_categories = []
             model_selected = self.selected_categories.get_model()
@@ -256,17 +257,16 @@ class ContentsPage(AssistantPage):
 
             return selected_categories
 
-
     def check_selection(self, *args):
-        if not self.is_text_exported() and not self.get_categories():
-            error = _('If export text is not selected, you have to select at least one tag.')
+        if self.is_filtered() and not self.get_categories():
+            error = _('When filtering by tags, you have to select at least one tag.')
             self.set_error_text(error)
             correct = False
         else:
             self.set_error_text('')
             correct = True
 
-        select = self.sel_categories_button.get_active()
+        select = self.filter_tags_button.get_active()
         self.available_categories.set_sensitive(select)
         self.selected_categories.set_sensitive(select)
         self.select_button.set_sensitive(select)
@@ -305,6 +305,7 @@ class SummaryPage(AssistantPage):
 class ExportAssistant(Assistant):
     def __init__(self, *args, **kwargs):
         Assistant.__init__(self, *args, **kwargs)
+        self.set_modal(True)
 
         self.exporters = get_exporters()
 
@@ -349,14 +350,19 @@ class ExportAssistant(Assistant):
 
         self.exporter = None
         self.path = None
+        self.set_forward_page_func(self.pageforward)
 
+    def pageforward(self, page):
+        if page == 2 and self.page2.export_selected_text():
+            return 4
+        else:
+            return page + 1
 
     def run(self):
         self.page2.refresh_dates()
         self.page3.refresh_categories_list()
         self.show_all()
 
-
     def _on_close(self, assistant):
         '''
         Do the export
@@ -364,7 +370,6 @@ class ExportAssistant(Assistant):
         self.hide()
         self.export()
 
-
     def _on_prepare(self, assistant, page):
         '''
         Called when a new page should be prepared, before it is shown
@@ -372,9 +377,15 @@ class ExportAssistant(Assistant):
         if page == self.page2:
             # Date Range
             self.exporter = self.page1.get_selected_object()
+            self.page2.prepare()
         elif page == self.page3:
             # Categories
-            pass
+            start_date, end_date = self.page2.get_date_range()
+            if not self.page2.export_all_days() and start_date == end_date:
+                self.page3.filter_tags_button.set_active(False)
+                self.page3.filter_tags_button.set_sensitive(False)
+            else:
+                self.page3.filter_tags_button.set_sensitive(True)
         elif page == self.page4:
             # Path
             self.page4.prepare(self.exporter)
@@ -382,51 +393,69 @@ class ExportAssistant(Assistant):
             # Summary
             self.path = self.page4.get_selected_path()
             self.page5.prepare()
-            format = self.exporter.NAME
             self.export_all_days = self.page2.export_all_days()
-            self.is_text_exported = self.page3.is_text_exported()
+            self.export_selected_text = self.page2.export_selected_text()
+            self.is_filtered = self.page3.is_filtered()
             self.exported_categories = self.page3.get_categories()
 
-            self.page5.add_setting(_('Format'), format)
+            self.page5.add_setting(_('Format'), self.exporter.NAME)
             self.page5.add_setting(_('Export all days'), self.yes_no(self.export_all_days))
             if not self.export_all_days:
                 start_date, end_date = self.page2.get_date_range()
-                self.page5.add_setting(_('Start date'), start_date)
-                self.page5.add_setting(_('End date'), end_date)
-            is_text_exported = self.yes_no(self.is_text_exported)
-            self.page5.add_setting(_('Export text'), is_text_exported)
-            self.page5.add_setting(_('Selected tags'), ', '.join(self.exported_categories))
+                if start_date == end_date:
+                    self.page5.add_setting(_('Date'), start_date)
+                    self.page5.add_setting(_('Export selected text only'), self.yes_no(self.export_selected_text))
+                else:
+                    self.page5.add_setting(_('Start date'), start_date)
+                    self.page5.add_setting(_('End date'), end_date)
+            if self.export_selected_text:
+                self.exported_categories = []
+            else:
+                self.page5.add_setting(_('Include text'), self.yes_no(self.page3.is_text_included()))
+                self.page5.add_setting(_('Include tags'), self.yes_no(self.page3.is_tags_included()))
+            if self.is_filtered:
+                self.page5.add_setting(_('Filtered by tags'), ', '.join(self.exported_categories))
             self.page5.add_setting(_('Export path'), self.path)
 
-
     def yes_no(self, value):
         return _('Yes') if value else _('No')
 
-
     def get_export_string(self, format):
-        if self.export_all_days:
-            export_days = self.journal.days
+        if self.export_selected_text and self.page2.selected_text:
+            markup_string = self.page2.selected_text
         else:
-            export_days = self.journal.get_days_in_date_range(*self.page2.get_date_range())
+            if self.export_all_days:
+                export_days = self.journal.days
+            else:
+                export_days = self.journal.get_days_in_date_range(*self.page2.get_date_range())
 
-        selected_categories = self.exported_categories
-        logging.debug('Selected Categories for Export: %s' % selected_categories)
-        export_text = self.is_text_exported
+            selected_categories = self.exported_categories
+            logging.debug('Selected Categories for Inclusion: %s' % selected_categories)
 
-        markup_strings_for_each_day = []
-        for day in export_days:
             # Save selected date format
             date_format = self.page3.date_format.get_value()
             self.journal.config['exportDateFormat'] = date_format
-            date_string = dates.format_date(date_format, day.date)
-            day_markup = markup.get_markup_for_day(day, with_text=export_text,
-                                            categories=selected_categories,
-                                            date=date_string)
-            markup_strings_for_each_day.append(day_markup)
 
-        markup_string = ''.join(markup_strings_for_each_day)
+            markup_strings_for_each_day = []
+            for day in export_days:
+                include_day = True
+                if self.is_filtered:
+                    include_day = False
+                    catagory_pairs = day.get_category_content_pairs()
+                    for category in selected_categories:
+                        if category in catagory_pairs:
+                            include_day = True
+                if include_day:
+                    date_string = dates.format_date(date_format, day.date)
+                    day_markup = markup.get_markup_for_day(day, with_text=self.page3.is_text_included(),
+                                                           with_tags=self.page3.is_tags_included(),
+                                                           categories=selected_categories,
+                                                           date=date_string)
+                    markup_strings_for_each_day.append(day_markup)
 
-        return markup.convert(markup_string, format, options={'toc': 0})
+            markup_string = ''.join(markup_strings_for_each_day)
+
+        return self.journal.convert(markup_string, format, options={'toc': 0})
 
     def export(self):
         format = self.exporter.FORMAT
@@ -489,18 +518,12 @@ class Exporter(object):
 
     @property
     def DEFAULTPATH(self):
-        return os.path.join(os.path.expanduser('~'), 'RedNotebook-Export_%s.%s' %
-                                (datetime.date.today(), self.EXTENSION))
-
-
-
-
-
-
+        return os.path.join(os.path.expanduser('~'),
+            'RedNotebook-Export_%s.%s' % (datetime.date.today(), self.EXTENSION))
 
 
 class PlainTextExporter(Exporter):
-    NAME = 'Plain Text'
+    NAME = _('Plain Text')
     #DESCRIPTION = 'Export journal to a plain textfile'
     EXTENSION = 'txt'
     FORMAT = 'txt'
@@ -531,7 +554,7 @@ class PdfExporter(Exporter):
         if self.is_available():
             return ''
         else:
-            return '(' + _('requires pywebkitgtk') +')'
+            return '(' + _('requires pywebkitgtk') + ')'
 
     @classmethod
     def is_available(cls):
@@ -543,8 +566,6 @@ class PdfExporter(Exporter):
 def get_exporters():
     exporters = [PlainTextExporter, HtmlExporter, LatexExporter, PdfExporter]
 
-    #exporters = filter(lambda exporter: exporter.is_available(), exporters)
-
     # Instantiate importers
     exporters = map(lambda exporter: exporter(), exporters)
     return exporters
@@ -559,4 +580,3 @@ if __name__ == '__main__':
     assistant.set_position(gtk.WIN_POS_CENTER)
     assistant.run()
     gtk.main()
-
