@@ -18,7 +18,6 @@
 # -----------------------------------------------------------------------
 
 import os
-import sys
 import logging
 import platform
 
@@ -26,6 +25,7 @@ import gtk
 
 from rednotebook.gui.customwidgets import UrlButton, CustomComboBoxEntry
 from rednotebook.gui.customwidgets import ActionButton
+from rednotebook.gui import editor
 from rednotebook.util import filesystem, utils, dates
 from rednotebook import info
 
@@ -53,15 +53,15 @@ class Option(gtk.HBox):
 
 
 class TickOption(Option):
-    def __init__(self, text, name, default_value=None, tooltip=''):
+    def __init__(self, text, name, value=None, default=0, tooltip=''):
         Option.__init__(self, '', name, tooltip=tooltip)
 
         self.check_button = gtk.CheckButton(text)
 
-        if default_value is None:
-            self.check_button.set_active(Option.config.read(name, 0) == 1)
+        if value is None:
+            self.check_button.set_active(Option.config.read(name, default) == 1)
         else:
-            self.check_button.set_active(default_value)
+            self.check_button.set_active(value)
 
         self.pack_start(self.check_button, False)
 
@@ -82,7 +82,7 @@ class AutostartOption(TickOption):
         self.autostart_file = os.path.join(autostart_dir, 'rednotebook.desktop')
         autostart_file_exists = os.path.exists(self.autostart_file)
         TickOption.__init__(self, _('Load RedNotebook at startup'), None,
-                        default_value=autostart_file_exists)
+                            value=autostart_file_exists)
 
     def get_value(self):
         return self.check_button.get_active()
@@ -100,28 +100,18 @@ class AutostartOption(TickOption):
                 os.remove(self.autostart_file)
 
 
-#class TextOption(Option):
-#   def __init__(self, text, name):
-#       self.entry = gtk.Entry(30)
-#       self.entry.set_text(Option.config.read(name, ''))
-#
-#       Option.__init__(self, text, name, self.entry)
-#
-#   def get_value(self):
-#       return self.entry.get_text()
-
-class CsvTextOption(Option):
-    def __init__(self, text, option_name, **kwargs):
+class TextOption(Option):
+    def __init__(self, text, option_name, default='', **kwargs):
         Option.__init__(self, text, option_name, **kwargs)
 
         # directly read the string, not the list
-        values_string = Option.config.read(option_name, '')
+        value = Option.config.read(option_name, default)
 
         # Ensure that we have a string here
-        values_string = unicode(values_string)
+        value = unicode(value)
 
         self.entry = gtk.Entry()
-        self.entry.set_text(values_string)
+        self.entry.set_text(value)
 
         self.pack_start(self.entry, True)
 
@@ -129,13 +119,6 @@ class CsvTextOption(Option):
         return self.entry.get_text().decode('utf-8')
 
 
-#class TextAndButtonOption(TextOption):
-#   def __init__(self, text, name, button):
-#       TextOption.__init__(self, text, name)
-
-#       self.widget.pack_end(button, False, False)
-
-
 class ComboBoxOption(Option):
     def __init__(self, text, name, entries):
         Option.__init__(self, text, name)
@@ -182,62 +165,53 @@ class DateFormatOption(ComboBoxOption):
         self.preview.set_text(label_text)
 
 
-class FontSizeOption(ComboBoxOption):
+class FontOption(Option):
     def __init__(self, text, name):
-        sizes = range(6, 15) + range(16, 29, 2) + [32, 36, 40, 48, 56, 64, 72]
-        sizes = ['default'] + map(str, sizes)
+        Option.__init__(self, text, name, '')
 
-        ComboBoxOption.__init__(self, text, name, sizes)
+        self.dialog = None
 
-        # Set default size if not present
-        size = Option.config.read(name, -1)
+        self.font_name = Option.config.read(name, editor.DEFAULT_FONT)
 
-        if size == -1:
-            self.combo.set_active_text('default')
+        self.label = gtk.Label()
+        self.label.set_text(self.font_name)
+
+        self.button = gtk.Button(_('Choose font ...'))
+        self.button.connect('clicked', self.on_button_clicked)
+
+        self.pack_start(self.label, False)
+        self.pack_start(self.button, False)
+
+    def on_button_clicked(self, widget):
+        if not self.dialog:
+            self.dialog = gtk.FontSelectionDialog(_('Choose font'))
+
+            self.dialog.set_font_name(self.font_name)
+            self.dialog.set_modal(True)
+            self.dialog.set_transient_for(Option.main_window.options_manager.dialog.dialog)
+            self.dialog.connect("destroy", self.dialog_destroyed)
+            self.dialog.ok_button.connect(
+                "clicked", self.font_selection_ok)
+            self.dialog.cancel_button.connect_object(
+                "clicked", lambda window: window.destroy(), self.dialog)
+
+        if not (self.dialog.flags() & gtk.VISIBLE):
+            self.dialog.show()
         else:
-            self.combo.set_active_text(str(size))
+            self.dialog.destroy()
+            self.dialog = None
 
-        self.combo.set_editable(False)
-        self.combo.combo_box.set_wrap_width(3)
+    def dialog_destroyed(self, widget):
+        self.dialog = None
 
-        self.combo.connect('changed', self.on_combo_changed)
+    def font_selection_ok(self, widget):
+        self.font_name = self.dialog.get_font_name()
+        self.label.set_text(self.font_name)
+        Option.main_window.set_font(self.font_name)
+        self.dialog.destroy()
 
-    def on_combo_changed(self, widget):
-        '''Live update'''
-        size = self.get_string_value()
-        Option.main_window.set_font_size(size)
-
-    def get_string_value(self):
-        '''We use 0 and 1 internally for size options'''
-        size = self.combo.get_active_text()
-        if size == 'default':
-            return -1
-        try:
-            return int(size)
-        except ValueError:
-            return -1
-
-
-#class SpinOption(LabelAndWidgetOption):
-#   def __init__(self, text, name):
-#
-#       adj = gtk.Adjustment(10.0, 6.0, 72.0, 1.0, 10.0, 0.0)
-#       self.spin = gtk.SpinButton(adj)#, climb_rate=1.0)
-#       self.spin.set_numeric(True)
-#       self.spin.set_range(6,72)
-#       self.spin.set_sensitive(True)
-#       value = Option.config.read(name, -1)
-#       if value >= 0:
-#           self.spin.set_value(value)
-#
-#       LabelAndWidgetOption.__init__(self, text, name, self.spin)
-#
-#   def get_value(self):
-#       return self.spin.get_value()
-#
-#   def get_string_value(self):
-#       value = int(self.get_value())
-#       return value
+    def get_value(self):
+        return self.font_name
 
 
 class OptionsDialog(object):
@@ -291,23 +265,17 @@ class OptionsManager(object):
         # If this option is activated on a system without a systray, the
         # application keeps on running in the background after it has been
         # closed. The option can still be activated in the configuration file.
-        if sys.platform.startswith('win'):
+        if filesystem.HAS_TRAY:
             self.options.append(TickOption(_('Close to system tray'), 'closeToTray',
                 tooltip=_('Closing the window will send RedNotebook to the tray')))
 
-        able_to_spell_check = self.main_window.day_text_field.can_spell_check()
-        tooltip = (_('Underline misspelled words') if able_to_spell_check else
-                _('Requires gtkspell.') + ' ' +
-                _('This is included in the python-gtkspell or python-gnome2-extras package'))
-        spell_check_option = TickOption(_('Check Spelling'), 'spellcheck',
-                tooltip=tooltip)
-        if not sys.platform == 'win32':
-            self.options.append(spell_check_option)
-        spell_check_option.set_sensitive(able_to_spell_check)
+        # Automatic switching between preview and edit mode.
+        self.options.append(TickOption(_('Switch between edit and preview mode automatically'),
+                                       'autoSwitchMode', default=0))
 
         # Check for new version
-
         check_version_option = TickOption(_('Check for new version at startup'), 'checkForNewVersion')
+
         def check_version_action(widget):
             utils.check_new_version(self.main_window.journal, info.version)
             # Apply changes from dialog to options window
@@ -319,13 +287,17 @@ class OptionsManager(object):
         self.options.append(check_version_option)
 
         self.options.extend([
-                FontSizeOption(_('Font Size'), 'mainFontSize'),
-                DateFormatOption(_('Date/Time format'), 'dateTimeString'),
-                CsvTextOption(_('Exclude from clouds'), 'cloudIgnoreList',
-                                tooltip=_('Do not show those comma separated words in any cloud')),
-                CsvTextOption(_('Allow small words in clouds'), 'cloudIncludeList',
-                                tooltip=_('Allow those words with 4 letters or less in the text cloud')),
-                ])
+            # Use separate fonts since the preview often doesn't support the edit font.
+            FontOption(_('Edit font:'), 'mainFont'),
+            TextOption(_('Preview font:'), 'previewFont',
+                       default='Ubuntu, sans-serif',
+                       tooltip=_('Comma-separated font names')),
+            DateFormatOption(_('Date/Time format'), 'dateTimeString'),
+            TextOption(_('Exclude from cloud'), 'cloudIgnoreList',
+                       tooltip=_('Do not show these comma separated words in the word cloud')),
+            TextOption(_('Include small words in cloud'), 'cloudIncludeList',
+                       tooltip=_('Allow these words with 4 letters or less')),
+        ])
 
 
         self.add_all_options()
@@ -339,14 +311,11 @@ class OptionsManager(object):
             self.main_window.cloud.update_lists()
             self.main_window.cloud.update(force_update=True)
 
-            spell_check_enabled = self.config.read('spellcheck', 0)
-            self.main_window.day_text_field.enable_spell_check(spell_check_enabled)
-
             visible = (self.config.read('closeToTray', 0) == 1)
             self.main_window.tray_icon.set_visible(visible)
         else:
             # Reset some options
-            self.main_window.set_font_size(self.config.read('mainFontSize', -1))
+            self.main_window.set_font(self.config.read('mainFont', editor.DEFAULT_FONT))
 
         self.dialog.hide()
 
@@ -364,4 +333,3 @@ class OptionsManager(object):
             else:
                 # We don't save the autostart setting in the config file
                 option.set()
-
