@@ -23,25 +23,37 @@ if not hasattr(__builtin__, '_'):
     def _(string):
         return string
 
-version =           '1.4.0'
-author =            'Jendrik Seipp'
-authorMail =        'jendrikseipp@web.de'
-url =               'http://rednotebook.sourceforge.net'
-answers_url =       'https://answers.launchpad.net/rednotebook'
-translation_url =   'https://translations.launchpad.net/rednotebook/'
-bug_url =           'https://bugs.launchpad.net/rednotebook/+filebug'
+try:
+    import argparse
+    assert argparse  # silence pyflakes
+except ImportError:
+    from rednotebook.external import argparse
 
-developers =    ['Jendrik Seipp <jendrikseipp@web.de>']
+from rednotebook.util import filesystem
+
+
+version = '1.8.1'
+author = 'Jendrik Seipp'
+authorMail = 'jendrikseipp@web.de'
+url = 'http://rednotebook.sourceforge.net'
+answers_url = 'https://answers.launchpad.net/rednotebook'
+translation_url = 'https://translations.launchpad.net/rednotebook/'
+bug_url = 'https://bugs.launchpad.net/rednotebook/+filebug'
+
+developers = ['Jendrik Seipp <jendrikseipp@web.de>',
+              '',
+              'Contributors:',
+              'Alistair Marshall <thatscottishengineer@gmail.com>']
 
 comments = '''\
 RedNotebook is a graphical journal to keep track of notes and
-thoughts. It includes a calendar navigation, customizable
+thoughts. It includes calendar navigation, customizable
 templates, export functionality and word clouds. You can also
 format, tag and search your entries.
 '''
 
 license_text = '''\
-Copyright (c) 2009,2010,2011  Jendrik Seipp
+Copyright (c) 2009-2013  Jendrik Seipp
 
 RedNotebook is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -58,57 +70,57 @@ with RedNotebook; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 '''
 
-command_line_help = '''\
-RedNotebook %s
-
-The optional journal-path can be one of the following:
+journal_path_help = '''\
+(optional) Specify the directory storing the journal data.
+The journal argument can be one of the following:
  - An absolute path (e.g. /home/username/myjournal)
  - A relative path (e.g. ../dir/myjournal)
  - The name of a directory under $HOME/.rednotebook/ (e.g. myjournal)
 
-If the journal-path is omitted the last session's journal will be used.
-At the first program start this defaults to "$HOME/.rednotebook/data".
-''' % version
+If the journal argument is omitted then the last session's journal
+path will be used. At the first program start, this defaults to
+"$HOME/.rednotebook/data".
+'''
+
+def get_commandline_parser():
+    parser = argparse.ArgumentParser(
+        description=comments,
+        formatter_class=argparse.RawTextHelpFormatter)
+    parser.add_argument('--version', action='version',
+                        version='RedNotebook %s' % version)
+    if filesystem.HAS_TRAY:
+        parser.add_argument('-m', '--minimized', dest='minimized', action='store_true',
+                            help='start mimimized to system tray')
+    parser.add_argument('--date', dest='start_date',
+                        help='load specified date (format YYYY-MM-DD only)')
+    parser.add_argument('journal', nargs='?', help=journal_path_help)
+    return parser
+
 
-ideas = _('Ideas')
 tags = _('Tags')
-cool_stuff = _('Cool Stuff')
-movies = _('Movies')
 work = _('Work')
 free_time = _('Free time')
-documentation = _('Documentation')
 todo = _('Todo')
 done = _('Done')
 rtm = _('Remember the milk')
 dishes = _('Wash the dishes')
-check_mail = _('Check mail')
-monty_python_grail = _('Monty Python and the Holy Grail')
-team_meeting = _('Team meeting')
-
 
 greeting = _('Hello!')
-intro=_('Some example text has been added to help you start and '
-'you can erase it whenever you like.')
+intro = _('Some example text has been added to help you start and '
+          'you can erase it whenever you like.')
 ### Translators: "Help" -> noun
 help_par = _('The example text and more documentation is available under "Help" -> "Contents".')
-overview1 = _('The interface is divided into three parts:')
-### Translators: The location "left"
-overview21 = _('Left')
-overview22 = _('Navigation and search')
-### Translators: The location "center"
-overview31 = _('Center')
-overview32 = _('Text for a day')
-### Translators: The location "right"
-overview41 = _('Right')
-overview42 = _('Tags for this day')
 
 ### Translators: noun
 preview = _('Preview')
 preview1 = _('There are two modes in RedNotebook, the __edit__ mode and the __preview__ mode.')
-### Translators: Preview -> noun
-preview2 = _('Click on Preview above to see the difference.')
+preview2 = _('Click on Edit above to see the difference.')
 preview_par = ' '.join([preview1, preview2])
 
+tags1 = _('Tagging is easy.')
+tags2 = _('Just use #hashtags like on twitter.')
+tags_par = ' '.join([tags1, tags2])
+
 example_entry = _('Today I went to the //pet shop// and bought a **tiger**. '
 'Then we went to the --pool-- park and had a nice time playing '
 'ultimate frisbee. Afterwards we watched "__Life of Brian__".')
@@ -138,16 +150,12 @@ goodbye_par = _('Have a nice day!')
 completeWelcomeText = '''\
 %(greeting)s %(intro)s %(help_par)s
 
-%(overview1)s
-
-- **%(overview21)s**: %(overview22)s
-- **%(overview31)s**: %(overview32)s
-- **%(overview41)s**: %(overview42)s
-
-
 === %(preview)s ===
 %(preview_par)s
 
+=== %(tags)s ===
+%(tags_par)s
+
 === %(save)s ===
 %(save_par)s
 
@@ -156,98 +164,33 @@ completeWelcomeText = '''\
 %(goodbye_par)s''' % globals()
 
 
-welcome_day = {'text': completeWelcomeText,
-cool_stuff: {_('Ate **two** cans of spam'): None},
-ideas: {_('Use a cool journal app'): None},
-movies: {_("Monty Python's Life of Brian"): None},
-documentation: None,
-}
+welcome_day = {'text': completeWelcomeText}
 
-example_day1 = {
-'text': '''\
-=== %(tags)s ===
-Besides the main text you can add tags to each day. A tag can have subtags as
-well. On the right you find some examples of tags.
+multiple_entries_text = _('''\
+=== Multiple entries ===
+You can add multiple entries to a single day by \
+using different journals (one named "Work", the other "Family"), \
+separating your entries with different titles (=== Work ===, === Family ===) \
+and using horizontal separator lines (20 “=”s).''')
 
-As you can see you can add the tag %(movies)s and fill it with the movies
-you watch on the respective days. Similarly you can record the things you do
-at work. If you just want to note that you went to work on a particular day,
-you can omit the "%(team_meeting)s" entry.
-
-Tags can be formatted **bold**, //italic//, etc.''' % globals(),
-movies: {monty_python_grail: None},
-documentation: None,
-work: {team_meeting: None},
-}
-
-multiple_entries_text = '''\
-=== Multiple Entries ===
-You can add multiple entries to one day in two ways:
-- Use two different journals (one named "%(work)s", the other "%(free_time)s")
-- Separate your two entries by different titles (===%(work)s===, ===%(free_time)s===)
-- Use a horizontal separator line (20 “=”s)
-
-
-''' % globals()
-
-multiple_entries_example = '''\
-====================
-
-=== %(work)s ===
-Here goes the first entry.
+multiple_entries_example = _('''\
+=== Work ===
+Here goes the first entry. It is about #work.
 
 ====================
 
-=== %(free_time)s ===
-Here comes the entry about the fun stuff.
-''' % globals()
+=== Family ===
+Here comes the entry about my #family.''')
 
-example_day2 = {
-'text': multiple_entries_text + multiple_entries_example,
-documentation: None, work: None, free_time: None}
+multiple_entries_day = {
+    'text': multiple_entries_text + '\n\n' + 20 * '=' + '\n\n' + multiple_entries_example}
 
-example_day3 = {
-'text': '''\
-=== Todo list ===
-You can also use RedNotebook as a todo list. An advantage is, that you never
-have to explicitly state the date when you added the todo item, you just add it
-on one day and it remains there until you delete it.
-
-Here is how it works:
-- On the right click on "Add Tag"
-- Fill "%(todo)s" and "Remember the milk" in the fields and hit "OK"
-- In the cloud on the left you can now click on "%(todo)s" and see all your todo items
-- This list can be sorted by day or by todo item if you click on "Date" or "Text" in the header
-
-
-- To tick off a todo item you can strike it out by adding "--" around the item.
-- To mark an item as important, add "**" around it.
-
-
-So --%(rtm)s-- becomes struck through and **%(dishes)s** becomes bold.
-
-Once you've finished an item, you could also change its tag name from
-"%(todo)s" to "%(done)s".''' % globals(),
-documentation: None,
-todo: {u'--%s--' % rtm: None,
-       u'**%s**' % dishes: None},
-done: {u'%s' % check_mail: None,},
-}
-
-example_content = [welcome_day, example_day1, example_day2, example_day3]
-
-ann_help_text = example_day1['text'].replace('===', '==')
-todo_help_text = example_day3['text']
+example_content = [welcome_day, multiple_entries_day]
 
+commandline_help = get_commandline_parser().format_help()
 
 help_text = '''
 == Layout ==
-%(overview1)s
-- **%(overview21)s**: %(overview22)s
-- **%(overview31)s**: %(overview32)s
-- **%(overview41)s**: %(overview42)s
-
-
 %(preview1)s
 
 == Text ==
@@ -276,7 +219,20 @@ instead of "-" you can create a **numbered list**.
 
 ```
 
-%(ann_help_text)s
+== Hashtags ==
+%(tags_par)s
+
+== Advanced tagging ==
+Until #hashtags were introduced, you could only tag a day with the tag panel on
+the right side, that is now hidden by default. Drag the slider to the left to
+see it.
+
+It provides an advanced tagging mechanism, allowing you to add a tags with
+subtags like Movies->James Bond. Apart from the fact that you can't have spaces
+in hashtags you can however achieve a similar effect only with hashtags by
+adding #Movies and #James_Bond to the day's text.
+
+Tags and subtags can be formatted **bold**, //italic//, etc.
 
 == Images, Files and Links ==
 RedNotebook lets you insert images, files and links into your entries.
@@ -285,7 +241,8 @@ above the main text field. The text will be inserted at the current
 cursor position.
 
 With the insert button you cannot insert **links to directories** on your computer.
-Those can be inserted manually however (``""[Home ""file:///home/""]""``).
+Those can be inserted manually however (``[Home ""file:///home/""]``
+becomes [Home ""file:///home/""]).
 
 == %(templates)s ==
 %(temp_par)s
@@ -298,27 +255,42 @@ from inside RedNotebook by opening the menu next to the "Template" button.
 On the left you find the search box. Double-clicking on a day in the search
 results lets you jump to it.
 
+You can search for text or dates (e.g. 2014, 2014-01, 2014-01-19).
+
 == Clouds ==[clouds]
+The 42 most frequently used words will appear in the word cloud on the left. Its
+contents are only refreshed after a program restart and when the journal is saved.
+
 If a word appears in the cloud that you don't want to see there, just right-click
 and select to hide it.
 Alternatively you can open the Preferences dialog and add the word to the cloud blacklist.
 Short words with less than 5 letters can be white-listed there as well.
 [Regular expressions http://docs.python.org/library/re.html] are allowed in the
-lists.
+lists. If you want to hide words with special characters, you can escape them with
+a backslash: 3\\.50\\?
 
 You can **hide the word cloud** by adding the regular expression .* to the
 blacklist. This will filter out all words.
 
-
 == Spellcheck ==
-RedNotebook supports spellchecking your entries if you have
-python-gtkspell installed (Only available on Linux).
-To highlight all misspelled words in your entries, select the corresponding option in
-the preferences window.
+RedNotebook supports spellchecking your entries. On Linux this feature
+needs the package python-gtkspell. The feature can turned on and off by
+toggling the item under the "Edit" menu.
 
 Since gtkspell 2.0.15, you can select the spellchecking language by right-clicking on the
 main text area (in edit mode) and choosing it from the submenu "Languages".
 
+=== Adding custom dictionaries under Windows ===
+We use the dictionaries available from the
+[​openoffice extension download site http://extensions.services.openoffice.org/dictionaries].
+You need to download the appropriate language extension file(s)
+(files are openoffice extensions *.oxt, which are just zip files that
+contain additional data). Once you have downloaded a dictionary extension
+file, you can rename it so it has a .zip extension and then extract the
+*.dic and *.aff files in it to <RedNotebook Dir>\\share\\enchant\\myspell\\.
+If RedNotebook is running, it will need to be restarted for new dictionaries
+to be recognized.
+
 == Options ==
 Make sure you check out the customizable options in the Preferences dialog. You can
 open this dialog by clicking on the entry in the "Edit" menu.
@@ -372,7 +344,7 @@ Obviously you have to be connected to the internet to use that feature. Be sure
 data regularly if you plan to save your content remotely. There are always more pitfalls when
 an internet connection is involved.
 
-=== Dual Boot ===
+=== Dual boot ===
 Using RedNotebook from multiple operating systems on the same computer is
 also possible. Save your Journal with "Journal->Save As" in a directory
 all systems can access. Then on the other systems you can open the
@@ -399,6 +371,14 @@ portable mode.
 To **activate portable mode**, change into the files/ directory and in the
 default.cfg file set portable=1.
 
+== Network drive ==
+Unfortunately, you cannot add links to files on network shares directly
+with the file selection dialog
+([Bug on launchpad ""https://bugs.launchpad.net/ubuntu/+source/gtk+2.0/+bug/304345""]).
+However, it is possible to enter links directly, for example
+``[U: ""file:///U:/""]`` to reference the mapped drive letter
+[U ""file:///U:/""].
+
 == Convert Latex output to PDF ==
 
 In recent RedNotebook versions you can export your journal directly to PDF,
@@ -438,15 +418,18 @@ open the .tex file with Texniccenter and select "Build Output" from the
 "Output" menu. The program will then create the beautifully looking PDF in the
 same directory.
 
-== Keyboard Shortcuts ==
+== Keyboard shortcuts ==
 ||   Action          |   Shortcut             |
 | Preview (On/Off)   | <Ctrl> + P             |
 | Find               | <Ctrl> + F             |
 | Go back one day    | <Ctrl> + PageUp        |
 | Go forward one day | <Ctrl> + PageDown      |
+| Go to today        | <Alt> + Home (Pos1)    |
 | Insert link        | <Ctrl> + L             |
 | Insert date/time   | <Ctrl> + D             |
 | New tag            | <Ctrl> + N             |
+| Export             | <Ctrl> + E             |
+| Fullscreen         | F11                    |
 
 You can find other shortcuts in the menus.
 
@@ -469,9 +452,29 @@ partition all RedNotebook data will be encrypted, too.
 == Tips ==
 %(multiple_entries_text)s
 
-%(todo_help_text)s
+=== Todo list ===
+You can also use RedNotebook as a todo list. An advantage is, that you never
+have to explicitly state the date when you added the todo item, you just add it
+on one day and it remains there until you delete it.
 
-=== Week Numbers ===
+Here is how it works:
+- Make sure the tag panel on the right is visible, if not drag the slider to the left.
+- On the right click on "Add Tag"
+- Fill "%(todo)s" and "Remember the milk" in the fields and hit "OK"
+- In the cloud on the left you can now click on "%(todo)s" and see all your todo items
+- This list can be sorted by day or by todo item if you click on "Date" or "Text" in the header
+
+
+- To tick off a todo item you can strike it out by adding "--" around the item.
+- To mark an item as important, add "**" around it.
+
+
+So --%(rtm)s-- becomes struck through and **%(dishes)s** becomes bold.
+
+Once you've finished an item, you could also change its tag name from
+"%(todo)s" to "%(done)s".
+
+=== Week numbers ===
 If you'd like to see the week numbers in the calendar, you can set the
 value of weekNumbers to 1 in the configuration file. This file
 normally resides at $HOME/.rednotebook/configuration.cfg
@@ -558,35 +561,16 @@ Formatting commands inside two pairs of "" are not interpreted (""**not bold**""
 Comments can be inserted after percent signs (**%%**). They will not be shown in the
 preview and the exports. The %% has to be the first character on the line.
 
-=== List of all Entries ===
-To get a list of all entries, just search for " " (the space character).
-This character is most likely included in all entries. You can sort the
+=== List of all entries ===
+To get a list of your entries search for "-". You can sort the
 resulting list chronologically by pressing the "Date" button.
 
 == Command line options ==
 ```
-Usage: rednotebook [options] [journal-path]
-
-RedNotebook %(version)s
-
-The optional journal-path can be one of the following:
- - An absolute path (e.g. /home/username/myjournal)
- - A relative path (e.g. ../dir/myjournal)
- - The name of a directory under $HOME/.rednotebook/ (e.g. myjournal)
-
-If the journal-path is omitted the last session's journal will be used.
-At the first program start this defaults to "$HOME/.rednotebook/data".
-
-
-Options:
-  -h, --help        show this help message and exit
-
-  -d, --debug       Output debugging messages (default: False)
-
-  -m, --minimized   Start mimimized to system tray (default: False)
+%(commandline_help)s
 ```
 
-== Data Format ==
+== Data format ==
 In this paragraph I will explain shortly what the RedNotebook files
 consist of. Firstly it is important to understand that the content
 is saved in a directory with many files, not just one file.
