@@ -17,8 +17,6 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import sys
 import datetime
 import os
@@ -26,12 +24,11 @@ import time
 import itertools
 import logging
 import locale
-from optparse import OptionParser
 from collections import defaultdict
 
 
 # Use basic stdout logging before we can initialize logging correctly
-logging.basicConfig(level=logging.INFO,
+logging.basicConfig(level=logging.DEBUG,
                     format='%(levelname)-8s %(message)s',
                     stream=sys.stdout)
 
@@ -39,6 +36,7 @@ logging.basicConfig(level=logging.INFO,
 # Allow importing from rednotebook package
 if hasattr(sys, "frozen"):
     from rednotebook.util import filesystem
+    assert filesystem  # silence pyflakes
 else:
     from util import filesystem
 
@@ -46,13 +44,11 @@ else:
 base_dir = os.path.abspath(os.path.join(filesystem.app_dir, '../'))
 sys.path.insert(0, base_dir)
 
-#from rednotebook.util import filesystem # creates a copy of filesystem module
-#import rednotebook.util.filesystem      # imports the original filesystem module
-from rednotebook.external import elibintl
-
 
 ## ---------------------- Enable i18n -------------------------------
 
+from rednotebook.external import elibintl
+
 # We need to translate 3 different types of strings:
 # * sourcecode strings
 # * gtkbuilder strings
@@ -60,9 +56,7 @@ from rednotebook.external import elibintl
 
 LOCALE_PATH = filesystem.get_utf8_path(filesystem.locale_dir)
 
-# the name of the gettext domain. because we have our translation files
-# not in a global folder this doesn't really matter, setting it to the
-# application name is a good idea tough.
+# the name of the gettext domain.
 GETTEXT_DOMAIN = 'rednotebook'
 
 # Register _() as a global translation function and set up the translation
@@ -70,53 +64,40 @@ try:
     elibintl.install(GETTEXT_DOMAIN, LOCALE_PATH)
 except locale.Error, err:
     # unsupported locale setting
-    logging.error('Locale could not be set: "%s"' %  err)
+    logging.error('Locale could not be set: "%s"' % err)
     logging.error('Probably you have to install the appropriate language packs')
+    # Make the _() function available even if gettext is not working.
+    import __builtin__
+    if not hasattr(__builtin__, '_'):
+        __builtin__.__dict__['_'] = lambda s: s
 
 ## ------------------- end Enable i18n -------------------------------
 
 
 from rednotebook.util import utils
+from rednotebook.util import markup
 from rednotebook import info
 from rednotebook import configuration
+from rednotebook import data
 
 
-
-def parse_options():
-    parser = OptionParser(usage="usage: %prog [options] [journal-path]",
-                          description=info.command_line_help,
-                          version="RedNotebook %s" % info.version,
-                          formatter=utils.IndentedHelpFormatterWithNL(),
-                          )
-
-    parser.add_option('-d', '--debug', dest='debug', default=False,
-                      action='store_true', help='Output debugging messages'
-                      ' (default: False)')
-
-    parser.add_option('-m', '--minimized', dest='minimized', default=False,
-                      action='store_true', help='Start mimimized to system tray'
-                      ' (default: False)')
-
-    options, args = parser.parse_args()
-
-    return options, args
-
-options, args = parse_options()
-
-
+args = info.get_commandline_parser().parse_args()
 
 ## ---------------------- Enable logging -------------------------------
 
 def setup_logging(log_file):
     file_logging_stream = open(log_file, 'w')
 
-    # We want to have the error messages in the logfile. In the frozen
-    # version we cannot log to sys.stderr because it's broken on
-    # windows. Stdout is redirected into a blackhole by py2exe.
+    # We want to have all stdout and stderr messages in the logfile.
+    # In the frozen version we cannot log to sys.stderr because it's
+    # broken on windows. Stdout doesn't work either it seems.
     stderr_streams = [file_logging_stream]
-    if not hasattr(sys, 'frozen'):
+    stdout_streams = [file_logging_stream]
+    if not filesystem.main_is_frozen():
         stderr_streams.append(sys.__stderr__)
+        stdout_streams.append(sys.__stdout__)
     sys.stderr = utils.StreamDuplicator(stderr_streams)
+    sys.stdout = utils.StreamDuplicator(stdout_streams)
 
     root_logger = logging.getLogger('')
     root_logger.setLevel(logging.DEBUG)
@@ -126,30 +107,16 @@ def setup_logging(log_file):
     for handler in root_logger.handlers:
         root_logger.removeHandler(handler)
 
-    # define a Handler which writes DEBUG messages or higher to the logfile
-    filelog = logging.StreamHandler(file_logging_stream)
-    filelog.setLevel(logging.DEBUG)
-    filelog_formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
-    # tell the handler to use this format
-    filelog.setFormatter(filelog_formatter)
-    # add the handler to the root logger
-    root_logger.addHandler(filelog)
-
-    level = logging.INFO
-    if options.debug:
-        level = logging.DEBUG
-
-    # define a Handler which writes INFO messages or higher to sys.stdout
+    # define a Handler which writes messages to sys.stdout
     console = logging.StreamHandler(sys.stdout)
-    console.setLevel(level)
+    console.setLevel(logging.DEBUG)
     # set a format which is simpler for console use
-    formatter = logging.Formatter('%(levelname)-8s %(message)s')
+    formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
     # tell the handler to use this format
     console.setFormatter(formatter)
     # add the handler to the root logger
     root_logger.addHandler(console)
 
-    logging.debug('sys.stdout logging level: %s' % level)
     logging.info('Writing log to file "%s"' % log_file)
 
 
@@ -161,6 +128,8 @@ setup_logging(dirs.log_file)
 
 ## ------------------ end Enable logging -------------------------------
 
+logging.info('System encoding: %s' % filesystem.ENCODING)
+logging.info('Language code: %s' % filesystem.LANGUAGE)
 
 try:
     import pygtk
@@ -177,8 +146,8 @@ try:
     # only gtk.gdk.threads_init(): pdf export works, but gui hangs afterwards
     # only gobject.threads_init(): pdf export works, gui works
     # both: pdf export works, gui hangs afterwards
-    gobject.threads_init() # only initializes threading in the glib/gobject module
-    #gtk.gdk.threads_init() # also initializes the gdk threads
+    gobject.threads_init()  # only initializes threading in the glib/gobject module
+    #gtk.gdk.threads_init()  # also initializes the gdk threads
 except (ImportError, AssertionError), e:
     logging.error(e)
     logging.error('gtk not found. Please install PyGTK (python-gtk2)')
@@ -209,15 +178,12 @@ class Journal:
 
         logging.info('Running in portable mode: %s' % self.dirs.portable)
 
-        self.testing = False
-        if options.debug:
-            self.testing = True
-            logging.debug('Debug Mode is on')
-
         # Allow starting minimized to tray
         # When we start minimized we have to set the tray icon visible
-        self.start_minimized = options.minimized
-        if self.start_minimized:
+        self.start_minimized = filesystem.HAS_TRAY and args.minimized
+        if not filesystem.HAS_TRAY:
+            self.config['closeToTray'] = 0
+        elif self.start_minimized:
             self.config['closeToTray'] = 1
 
         self.month = None
@@ -237,13 +203,20 @@ class Journal:
 
         utils.set_environment_variables(self.config)
 
-        self.actual_date = datetime.date.today()
+        self.actual_date = self.get_start_date()
 
         # Let components check if the MainWindow has been created
         self.frame = None
         self.frame = MainWindow(self)
 
-        self.open_journal(self.get_journal_path())
+        journal_path = self.get_journal_path()
+        if not self.dirs.is_valid_journal_path(journal_path):
+            logging.error('Invalid directory: %s. Using default journal.' % journal_path)
+            self.show_message(_('You cannot use this directory for your journal:') +
+                              ' %s' % journal_path + '. ' + _('Opening default journal.'),
+                              error=True)
+            journal_path = self.dirs.default_data_dir
+        self.open_journal(journal_path)
 
         self.archiver = backup.Archiver(self)
         #self.archiver.check_last_backup_date()
@@ -253,8 +226,7 @@ class Journal:
             utils.check_new_version(self, info.version, startup=True)
 
         # Automatically save the content after a period of time
-        if not self.testing:
-            gobject.timeout_add_seconds(600, self.save_to_disk)
+        gobject.timeout_add_seconds(600, self.save_to_disk)
 
 
     def get_journal_path(self):
@@ -262,7 +234,7 @@ class Journal:
         Retrieve the path from optional args or return standard value if args
         not present
         '''
-        if not args:
+        if not args.journal:
             data_dir = self.config.read('dataDir', self.dirs.data_dir)
             if not os.path.isabs(data_dir):
                 data_dir = os.path.join(self.dirs.app_dir, data_dir)
@@ -273,7 +245,7 @@ class Journal:
         # or an absolute path /home/username/myjournal
         # Try to find the journal under the standard location or at the given
         # absolute or relative location
-        path_arg = args[0]
+        path_arg = args.journal
 
         logging.debug('Trying to find journal "%s"' % path_arg)
 
@@ -285,18 +257,28 @@ class Journal:
                     return path
                 else:
                     logging.warning('To open a journal you must specify a '
-                                'directory, not a file.')
+                                    'directory, not a file.')
 
-        logging.error('The path "%s" is no valid journal directory. '
-                    'Execute "rednotebook -h" for instructions' % path_arg)
+        logging.error('The path "%s" is not a valid journal directory. '
+                      'Execute "rednotebook -h" for instructions' % path_arg)
         sys.exit(1)
 
+    def get_start_date(self):
+        '''
+        Retrieve the date from optional args or otherwise return 'today'
+        '''
+        if not args.start_date:
+            return datetime.date.today()
+
+        try:
+            return dates.get_date_from_date_string(args.start_date)
+        except ValueError:
+            logging.error('Invalid date: %s (required format: YYYY-MM-DD).' % args.start_date)
+            sys.exit(2)
 
     def exit(self):
         self.frame.add_values_to_config()
 
-        self.config['running'] = 0
-
         # Make it possible to stop the program from exiting
         # e.g. if the journal could not be saved
         self.is_allowed_to_exit = True
@@ -309,29 +291,29 @@ class Journal:
             logging.shutdown()
             gtk.main_quit()
 
+    def convert(self, text, target, headers=None, options=None):
+        options = options or {}
+        options['font'] = self.config.read('previewFont', 'Ubuntu, sans-serif')
+        return markup.convert(text, target, self.dirs.data_dir, headers=headers, options=options)
 
     def save_to_disk(self, exit_imminent=False, changing_journal=False, saveas=False):
-        #logging.info('Trying to save the journal')
-
         self.save_old_day()
 
         try:
             filesystem.make_directory(self.dirs.data_dir)
-        except OSError:
-            self.frame.show_save_error_dialog(exit_imminent)
-            return True
-
-        if not os.path.exists(self.dirs.data_dir):
-            logging.error('Save path does not exist')
+        except (OSError, IOError):
             self.frame.show_save_error_dialog(exit_imminent)
             return True
 
         something_saved = storage.save_months_to_disk(self.months,
-            self.dirs.data_dir, self.frame, exit_imminent, changing_journal, saveas)
+            self.dirs.data_dir, self.frame, exit_imminent, saveas)
 
         if something_saved:
             self.show_message(_('The content has been saved to %s') % self.dirs.data_dir, error=False)
             logging.info('The content has been saved to %r' % self.dirs.data_dir)
+        elif something_saved is None:
+            # Don't display this as an error, because we already show a dialog.
+            self.show_message(_('The journal could not be saved'), error=False)
         else:
             self.show_message(_('Nothing to save'), error=False)
 
@@ -346,28 +328,15 @@ class Journal:
 
 
     def open_journal(self, data_dir):
+        if not os.path.exists(data_dir):
+            logging.warning('The dir %s does not exist. Select a different dir.'
+                            % data_dir)
+            return
 
         if self.months:
             self.save_to_disk(changing_journal=True)
 
-        # Password Protection
-        #password = self.config.read('password', '')
-
         logging.info('Opening journal at %r' % data_dir)
-
-        if not os.path.exists(data_dir):
-            logging.warning('The data dir %s does not exist. Select a different dir.'
-                        % data_dir)
-
-            self.frame.show_dir_chooser('open', dir_not_found=True)
-            return
-
-        data_dir_empty = not os.listdir(data_dir)
-
-        if data_dir_empty:
-            self.show_message(_('The selected folder is empty. A new journal has been created.'),
-                                error=False)
-
         self.dirs.data_dir = data_dir
 
         self.month = None
@@ -380,7 +349,7 @@ class Journal:
 
         self.stats = Statistics(self)
 
-        if self.is_first_start and data_dir_empty and len(self.days) == 0:
+        if self.is_first_start and not os.listdir(data_dir) and len(self.days) == 0:
             self.add_instruction_content()
 
         self.frame.cloud.update(force_update=True)
@@ -396,11 +365,7 @@ class Journal:
         self.title = filesystem.get_journal_title(data_dir)
 
         # Set frame title
-        if self.title == 'data':
-            frame_title = 'RedNotebook'
-        else:
-            frame_title = 'RedNotebook - ' + self.title
-        self.frame.main_frame.set_title(frame_title)
+        self.set_frame_title()
 
         # Save the folder for next start
         if not self.dirs.portable:
@@ -409,6 +374,12 @@ class Journal:
             rel_data_dir = filesystem.get_relative_path(self.dirs.app_dir, data_dir)
             self.config['dataDir'] = rel_data_dir
 
+    def set_frame_title(self):
+        parts = ['RedNotebook']
+        if self.title != 'data':
+            parts.append(self.title)
+        parts.append(dates.format_date('%x', self.date))
+        self.frame.main_frame.set_title(' - '.join(parts))
 
     def get_month(self, date):
         '''
@@ -419,7 +390,7 @@ class Journal:
         year_and_month = dates.get_year_and_month_from_date(date)
 
         # Selected month has not been loaded or created yet
-        if not year_and_month in self.months:
+        if year_and_month not in self.months:
             self.months[year_and_month] = Month(date.year, date.month)
 
         return self.months[year_and_month]
@@ -431,7 +402,7 @@ class Journal:
         self.day.content = self.frame.categories_tree_view.get_day_content()
         self.day.text = self.frame.get_day_text()
 
-        content_changed = not (old_content == self.day.content)
+        content_changed = (old_content != self.day.content)
         if content_changed:
             self.month.edited = True
 
@@ -448,6 +419,8 @@ class Journal:
 
         self.frame.set_date(self.month, self.date, self.day)
 
+        self.set_frame_title()
+
 
     def merge_days(self, days):
         '''
@@ -491,15 +464,23 @@ class Journal:
         self.change_date(prev_date)
 
 
-    def show_message(self, message_text, error=False, countdown=True):
-        self.frame.statusbar.show_text(message_text, error, countdown)
+    def show_message(self, msg, title=None, error=False):
+        if error and not title:
+            title = _('Error')
+        if error:
+            msg_type = gtk.MESSAGE_ERROR
+            log_level = logging.ERROR
+        else:
+            msg_type = gtk.MESSAGE_INFO
+            log_level = logging.INFO
+        self.frame.show_message(title, msg, msg_type)
+        logging.log(log_level, '%s. %s' % (title, msg))
 
 
     @property
     def categories(self):
         return list(sorted(set(itertools.chain.from_iterable(
-                                        day.categories for day in self.days)),
-                           key=utils.sort_asc))
+            day.categories for day in self.days)), cmp=locale.strcoll))
 
 
     def normalize_tag(self, tag):
@@ -526,7 +507,7 @@ class Journal:
             return self.days
         days = []
         for day in self.days:
-            day_tags = [tag.replace(' ', '').lower() for tag in day.categories]
+            day_tags = set(data.escape_tag(tag) for tag in day.categories)
             if all(tag in day_tags for tag in tags):
                 days.append(day)
         return days
@@ -536,6 +517,8 @@ class Journal:
         '''
         Returns a dictionary mapping the words to their number of appearance
         '''
+        # TODO: Use collections.Counter in Python2.7
+        # TODO: Check if concatenating all text and using a regex is faster.
         word_dict = defaultdict(int)
         for day in self.days:
             words = day.get_words()
@@ -619,4 +602,3 @@ def main():
 
 if __name__ == '__main__':
     main()
-
