@@ -17,14 +17,13 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-# For Python 2.5 compatibility.
-from __future__ import with_statement
-
-import os
-import sys
-import logging
-import re
 import codecs
+import logging
+import os
+import re
+import stat
+import sys
+
 
 try:
     import yaml
@@ -36,6 +35,7 @@ except ImportError:
 try:
     from yaml import CLoader as Loader
     from yaml import CSafeDumper as Dumper
+    assert Loader and Dumper  # silence pyflakes
 except ImportError:
     from yaml import Loader, Dumper
     logging.info('Using pyyaml for loading and dumping')
@@ -43,6 +43,22 @@ except ImportError:
 from rednotebook.data import Month
 
 
+def get_journal_files(data_dir):
+    # Format: 2010-05.txt
+    date_exp = re.compile(r'(\d{4})-(\d{2})\.txt$')
+
+    for file in sorted(os.listdir(data_dir)):
+        match = date_exp.match(file)
+        if match:
+            year = int(match.group(1))
+            month = int(match.group(2))
+            assert month in range(1, 12 + 1)
+            path = os.path.join(data_dir, file)
+            yield (path, year, month)
+        else:
+            logging.debug('%s is not a valid month filename' % file)
+
+
 def _load_month_from_disk(path, year_number, month_number):
     '''
     Load the month file at path and return a month object
@@ -61,9 +77,9 @@ def _load_month_from_disk(path, year_number, month_number):
     except IOError:
         #If that fails, there is nothing to load, so just display an error message
         logging.error('Error: The file %s could not be read' % path)
-    except Exception, err:
+    except Exception:
         logging.error('An error occured while reading %s:' % path)
-        logging.error('%s' % err)
+        raise
     # If we continued here, the possibly corrupted file would be overwritten
     sys.exit(1)
 
@@ -71,38 +87,21 @@ def _load_month_from_disk(path, year_number, month_number):
 def load_all_months_from_disk(data_dir):
     '''
     Load all months and return a directory mapping year-month values
-    to month objects
+    to month objects.
     '''
-    # Format: 2010-05.txt
-    date_exp = re.compile(r'(\d{4})-(\d{2})\.txt$')
-
     months = {}
 
     logging.debug('Starting to load files in dir "%s"' % data_dir)
-    files = sorted(os.listdir(data_dir))
-    for file in files:
-        match = date_exp.match(file)
-        if match:
-            year_string = match.group(1)
-            month_string = match.group(2)
-            year_month = year_string + '-' + month_string
+    for path, year_number, month_number in get_journal_files(data_dir):
+        month = _load_month_from_disk(path, year_number, month_number)
+        if month:
+            months['%d-%02d' % (year_number, month_number)] = month
 
-            year_number = int(year_string)
-            month_number = int(month_string)
-            assert month_number in range(1, 13)
-
-            path = os.path.join(data_dir, file)
-
-            month = _load_month_from_disk(path, year_number, month_number)
-            if month:
-                months[year_month] = month
-        else:
-            logging.debug('%s is not a valid month filename' % file)
     logging.debug('Finished loading files in dir "%s"' % data_dir)
     return months
 
 
-def save_months_to_disk(months, dir, frame, exit_imminent=False, changing_journal=False, saveas=False):
+def save_months_to_disk(months, dir, frame, exit_imminent=False, saveas=False):
     '''
     Do the actual saving and return if something has been saved
     '''
@@ -126,11 +125,15 @@ def save_months_to_disk(months, dir, frame, exit_imminent=False, changing_journa
                 try:
                     # This version produces readable unicode and no python directives
                     yaml.dump(month_content, month_file, Dumper=Dumper, allow_unicode=True)
-                    #yaml.safe_dump(month_content, month_file, allow_unicode=True)
                     month.edited = False
                     logging.debug('Wrote file %s' % month_file_string)
-                except OSError:
-                    frame.show_save_error_dialog(exit_imminent)
-                except IOError:
+                except (OSError, IOError):
                     frame.show_save_error_dialog(exit_imminent)
+                    return None
+            try:
+                # Make file readable and writable only by the owner.
+                os.chmod(month_file_string, stat.S_IRUSR | stat.S_IWUSR)
+            except OSError:
+                pass
+
     return something_saved
