@@ -17,54 +17,47 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import os
-import imp
 import sys
 import locale
 import subprocess
 import logging
 import codecs
 import webbrowser
-from glob import glob
 
 ENCODING = sys.getfilesystemencoding() or locale.getlocale()[1] or 'UTF-8'
+LANGUAGE = locale.getdefaultlocale()[0]
+REMOTE_PROTOCOLS = ['http', 'ftp', 'irc']
+
+IS_WIN = sys.platform.startswith('win')
+HAS_TRAY = IS_WIN  # A smarter detection is needed here ;)
 
 
 def get_unicode_path(path):
     return unicode(path, ENCODING)
 
 def get_utf8_path(path):
-    return path.encode(ENCODING)
+    return path.encode('UTF-8')
 
 
-#from http://www.py2exe.org/index.cgi/HowToDetermineIfRunningFromExe
 def main_is_frozen():
-    return (hasattr(sys, "frozen") or # new py2exe
-        hasattr(sys, "importers") # old py2exe
-        or imp.is_frozen("__main__")) # tools/freeze
-
-def get_main_dir():
-    if main_is_frozen():
-        return os.path.dirname(sys.executable)
-    return os.path.dirname(sys.argv[0])
-#------------------------------------------------------------------------------
+    return hasattr(sys, "frozen")
 
 
-if not main_is_frozen():
-    app_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../'))
-    app_dir = os.path.normpath(app_dir)
+if main_is_frozen():
+    app_dir = sys._MEIPASS  # os.path.dirname(sys.executable)
 else:
-    app_dir = get_main_dir()
-
+    app_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 app_dir = get_unicode_path(app_dir)
 
+if IS_WIN:
+    locale_dir = os.path.join(app_dir, 'share', 'locale')
+else:
+    locale_dir = os.path.join(sys.prefix, 'share', 'locale')
+
 image_dir = os.path.join(app_dir, 'images')
 frame_icon_dir = os.path.join(image_dir, 'rednotebook-icon')
 files_dir = os.path.join(app_dir, 'files')
-gui_dir = os.path.join(app_dir, 'gui')
-locale_dir = os.path.join(app_dir, 'i18n')
 
 user_home_dir = get_unicode_path(os.path.expanduser('~'))
 
@@ -95,6 +88,8 @@ class Filenames(dict):
         self.last_pic_dir = self.user_home_dir
         self.last_file_dir = self.user_home_dir
 
+        self.forbidden_dirs = [user_home_dir, self.journal_user_dir]
+
 
     def get_user_dir(self, config):
         custom = config.read('userDir', '')
@@ -115,13 +110,18 @@ class Filenames(dict):
         return user_dir
 
 
+    def is_valid_journal_path(self, path):
+        return os.path.isdir(path) and os.path.abspath(path) not in self.forbidden_dirs
+
+
     def __getattribute__(self, attr):
-        user_paths = dict((('template_dir', 'templates'),
-                        ('temp_dir', 'tmp'),
-                        ('default_data_dir', 'data'),
-                        ('config_file', 'configuration.cfg'),
-                        ('log_file', 'rednotebook.log'),
-                        ))
+        user_paths = dict((
+            ('template_dir', 'templates'),
+            ('temp_dir', 'tmp'),
+            ('default_data_dir', 'data'),
+            ('config_file', 'configuration.cfg'),
+            ('log_file', 'rednotebook.log'),
+        ))
 
         if attr in user_paths:
             return os.path.join(self.journal_user_dir, user_paths.get(attr))
@@ -139,6 +139,7 @@ def read_file(filename):
 
     try:
         import chardet
+        assert chardet  # silence pyflakes
     except ImportError:
         logging.info("chardet not found. 'utf-8' encoding will be assumed")
         chardet = None
@@ -223,9 +224,6 @@ def get_relative_path(from_dir, to_dir):
     else:
         return to_dir
 
-def get_icons():
-    return glob(os.path.join(frame_icon_dir, '*.png'))
-
 
 def get_journal_title(dir):
     '''
@@ -246,14 +244,12 @@ def get_platform_info():
     import yaml
 
     functions = [platform.machine, platform.platform, platform.processor,
-                platform.python_version, platform.release, platform.system,]
-    values = map(lambda function: function(), functions)
-    functions = map(lambda function: function.__name__, functions)
-    names_values = zip(functions, values)
+                 platform.python_version, platform.release, platform.system]
+    names_values = [(func.__name__, func()) for func in functions]
 
     lib_values = [('GTK version', gtk, 'gtk_version'),
-                    ('PyGTK version', gtk, 'pygtk_version'),
-                    ('Yaml version', yaml, '__version__'),]
+                  ('PyGTK version', gtk, 'pygtk_version'),
+                  ('Yaml version', yaml, '__version__')]
 
     for name, object, value in lib_values:
         try:
@@ -277,17 +273,20 @@ def system_call(args):
 def get_local_url(url):
     '''
     Sanitize url, make it absolute and normalize it, then add file://(/) scheme
+
+    Links and images only work in webkit on windows if the files have
+    file:/// (3 slashes) in front of the filename.
+    Strangely when clicking a link that has two slashes (file://C:\file.ext),
+    webkit returns the path file://C/file.ext .
     '''
     orig_url = url
-    if url.startswith('file:///') and sys.platform == 'win32':
+    if url.startswith('file:///') and IS_WIN:
         url = url.replace('file:///', '')
     if url.startswith('file://'):
         url = url.replace('file://', '')
     url = os.path.normpath(url)
 
-    import urllib
-    url = urllib.unquote(url).decode('utf-8')
-    scheme = 'file:///' if sys.platform == 'win32' else 'file://'
+    scheme = 'file:///' if IS_WIN else 'file://'
     url = scheme + url
     logging.debug('Transformed local URI %s to %s' % (orig_url, url))
     return url
@@ -301,6 +300,20 @@ def open_url_in_browser(url):
         logging.exception('Failed to open web browser')
 
 
+def unquote_url(url):
+    import urllib
+    return urllib.unquote(url).decode('utf-8')
+
+
+def _open_url_with_call(url, prog):
+    try:
+        logging.info('Trying to open %s with %s' % (url, prog))
+        system_call([prog, url])
+    except (OSError, subprocess.CalledProcessError):
+        logging.exception('Opening %s with %s failed' % (url, prog))
+        # If everything failed, try the webbrowser
+        open_url_in_browser(url)
+
 def open_url(url):
     '''
     Opens a file with the platform's preferred method
@@ -310,34 +323,20 @@ def open_url(url):
         return
 
     # Try opening the file locally
-    if sys.platform == 'win32':
+    if IS_WIN:
         try:
-            url = get_local_url(url)
+            url = unquote_url(url)
+            if url.startswith(u'file:') or os.path.exists(url):
+                url = get_local_url(url)
             logging.info('Trying to open %s with "os.startfile"' % url)
             # os.startfile is only available on windows
             os.startfile(url)
-            return
         except OSError:
             logging.exception('Opening %s with "os.startfile" failed' % url)
-
     elif sys.platform == 'darwin':
-        try:
-            logging.info('Trying to open %s with "open"' % url)
-            system_call(['open', url])
-            return
-        except OSError, subprocess.CalledProcessError:
-            logging.exception('Opening %s with "open" failed' % url)
-
+        _open_url_with_call(url, 'open')
     else:
-        try:
-            logging.info( 'Trying to open %s with xdg-open' % url)
-            system_call(['xdg-open', url])
-            return
-        except OSError, subprocess.CalledProcessError:
-            logging.exception('Opening %s with xdg-open failed' % url)
-
-    # If everything failed, try the webbrowser
-    open_url_in_browser(url)
+        _open_url_with_call(url, 'xdg-open')
 
 
 
